extern crate reqwest;
extern crate serde;
use serde::Deserialize;
use serenity::prelude::*;
use serenity::{
    framework::standard::{macros::command, Args, CommandResult},
    model::channel::Message,
    utils::Colour,
};

use std::collections::HashMap;

const DOG_URLS: [&'static str; 3] = [
    "https://dog.ceo/api/breed/{}/images/random",
    "https://dog.ceo/api/breeds/list",
    "https://dog.ceo/api/breeds/image/random",
];

async fn get_dog(url: &str) -> Result<String, reqwest::Error> {
    let resp = reqwest::get(url).await?;
    let jresp: HashMap<String, String> = resp.json().await?;
    let dog_url = &jresp["message"];
    Ok(dog_url.to_string())
}

#[command]
#[description = "sends a pic of a random pug"]
pub async fn pug(ctx: &Context, msg: &Message, _args: Args) -> CommandResult {
    let url = get_dog(&DOG_URLS[0].replace("{}", "pug")).await?;
    msg.channel_id
        .send_message(&ctx.http, |m| {
            m.embed(|e| e.image(url).colour(Colour::BLUE))
        })
        .await?;
    Ok(())
}

#[derive(Deserialize)]
struct DogList {
    #[allow(dead_code)]
    status: String,
    message: Vec<String>,
}

async fn get_dog_list() -> Result<DogList, reqwest::Error> {
    let resp = reqwest::get(DOG_URLS[1]).await?;
    let dlist: DogList = resp.json().await?;
    Ok(dlist)
}

#[command]
#[description = "sends a pic of a random dog of a given bread"]
pub async fn dog(ctx: &Context, msg: &Message, mut args: Args) -> CommandResult {
    match args.single::<String>() {
        Ok(ref breed) if breed == "list" => {
            let list_of_breads = get_dog_list().await?;
            msg.channel_id
                .say(
                    &ctx.http,
                    format!(
                        "**Here is a list of valid dog breeds**\n{:?}",
                        list_of_breads.message
                    ),
                )
                .await?;
        }
        Ok(bread) => {
            let url = get_dog(&DOG_URLS[0].replace("{}", &bread)).await?;
            msg.channel_id
                .send_message(&ctx.http, |m| {
                    m.embed(|e| e.image(url).colour(Colour::BLUE))
                })
                .await?;
        }
        Err(_) => {
            let url = get_dog(&DOG_URLS[2]).await?;
            msg.channel_id
                .send_message(&ctx.http, |m| {
                    m.embed(|e| e.image(url).colour(Colour::BLUE))
                })
                .await?;
        }
    }
    Ok(())
}
