use std::collections::HashSet;

use serenity::{
    client::Client,
    framework::standard::{
        help_commands,
        macros::{group, help},
        Args, CommandGroup, CommandResult, HelpOptions, StandardFramework,
    },
    model::prelude::*,
    prelude::*,
};

use std::env;
mod commands;
use commands::{
    AVATAR_COMMAND, CAT_RAT_CHEESE_COMMAND, DOG_COMMAND, ECHO_COMMAND, EIGHTBALL_COMMAND,
    PING_COMMAND, PROFILE_COMMAND, PUG_COMMAND, SPAM_COMMAND,
};
mod events;

#[help]
async fn my_help(
    ctx: &Context,
    msg: &Message,
    args: Args,
    help_options: &'static HelpOptions,
    groups: &[&'static CommandGroup],
    owners: HashSet<UserId>,
) -> CommandResult {
    help_commands::with_embeds(ctx, msg, args, &help_options, groups, owners).await;
    Ok(())
}

#[tokio::main]
async fn main() {
    let mut client = Client::builder(&env::var("DISCORD_TOKEN").expect("token"))
        .event_handler(events::Handler)
        .framework(
            StandardFramework::new()
                .configure(|c| c.prefix("&"))
                .help(&MY_HELP)
                .group(&FUN_GROUP)
                .group(&DISCORD_GROUP)
                .group(&UTILITY_GROUP)
                .group(&DOGS_GROUP)
                .group(&RUNESCAPE_GROUP),
        )
        .await
        .expect("Error creating client");
    if let Err(why) = client.start().await {
        println!("An error occurred while running the client: {:?}", why);
    }
}

#[group]
#[commands(cat_rat_cheese, echo, spam, eightball)]
struct Fun;

#[group]
#[commands(avatar)]
struct Discord;

#[group]
#[commands(ping)]
struct Utility;

#[group]
#[commands(pug, dog)]
struct Dogs;

#[group]
#[commands(profile)]
struct Runescape;
